@@ -6,30 +6,54 @@
 #          Roman Goj <roman.goj@gmail.com>
 #
 # License: BSD-3-Clause
+# Copyright the MNE-Python contributors.
 import numpy as np
 
+from .._fiff.pick import pick_channels, pick_info
 from ..channels import equalize_channels
-from ..io.pick import pick_info, pick_channels
-from ..utils import (logger, verbose, _check_one_ch_type,
-                     _check_channels_spatial_filter, _check_rank,
-                     _check_option, _validate_type, warn)
 from ..forward import _subject_from_forward
-from ..minimum_norm.inverse import combine_xyz, _check_reference, _check_depth
+from ..minimum_norm.inverse import _check_depth, _check_reference, combine_xyz
 from ..rank import compute_rank
-from ..source_estimate import _make_stc, _get_src_type
+from ..source_estimate import _get_src_type, _make_stc
 from ..time_frequency import EpochsTFR
 from ..time_frequency.tfr import _check_tfr_complex
-from ._compute_beamformer import (_prepare_beamformer_input,
-                                  _compute_beamformer, _check_src_type,
-                                  Beamformer, _compute_power,
-                                  _proj_whiten_data)
+from ..utils import (
+    _check_channels_spatial_filter,
+    _check_one_ch_type,
+    _check_option,
+    _check_rank,
+    _validate_type,
+    logger,
+    verbose,
+    warn,
+)
+from ._compute_beamformer import (
+    Beamformer,
+    _check_src_type,
+    _compute_beamformer,
+    _compute_power,
+    _prepare_beamformer_input,
+    _proj_whiten_data,
+)
 
 
 @verbose
-def make_dics(info, forward, csd, reg=0.05, noise_csd=None, label=None,
-              pick_ori=None, rank=None, weight_norm=None,
-              reduce_rank=False, depth=1., real_filter=True,
-              inversion='matrix', verbose=None):
+def make_dics(
+    info,
+    forward,
+    csd,
+    reg=0.05,
+    noise_csd=None,
+    label=None,
+    pick_ori=None,
+    rank=None,
+    weight_norm=None,
+    reduce_rank=False,
+    depth=1.0,
+    real_filter=True,
+    inversion="matrix",
+    verbose=None,
+):
     """Compute a Dynamic Imaging of Coherent Sources (DICS) spatial filter.
 
     This is a beamformer filter that can be used to estimate the source power
@@ -166,57 +190,74 @@ def make_dics(info, forward, csd, reg=0.05, noise_csd=None, label=None,
     .. footbibliography::
     """  # noqa: E501
     rank = _check_rank(rank)
-    _check_option('pick_ori', pick_ori,
-                  [None, 'vector', 'normal', 'max-power'])
-    _check_option('inversion', inversion, ['single', 'matrix'])
-    _validate_type(weight_norm, (str, None), 'weight_norm')
+    _check_option("pick_ori", pick_ori, [None, "vector", "normal", "max-power"])
+    _check_option("inversion", inversion, ["single", "matrix"])
+    _validate_type(weight_norm, (str, None), "weight_norm")
 
     frequencies = [np.mean(freq_bin) for freq_bin in csd.frequencies]
     n_freqs = len(frequencies)
 
-    _, _, allow_mismatch = _check_one_ch_type('dics', info, forward, csd,
-                                              noise_csd)
+    _, _, allow_mismatch = _check_one_ch_type("dics", info, forward, csd, noise_csd)
     # remove bads so that equalize_channels only keeps all good
-    info = pick_info(info, pick_channels(info['ch_names'], [], info['bads']))
+    info = pick_info(info, pick_channels(info["ch_names"], [], info["bads"]))
     info, forward, csd = equalize_channels([info, forward, csd])
 
     csd, noise_csd = _prepare_noise_csd(csd, noise_csd, real_filter)
 
-    depth = _check_depth(depth, 'depth_sparse')
-    if inversion == 'single':
-        depth['combine_xyz'] = False
+    depth = _check_depth(depth, "depth_sparse")
+    if inversion == "single":
+        depth["combine_xyz"] = False
 
-    is_free_ori, info, proj, vertices, G, whitener, nn, orient_std = \
-        _prepare_beamformer_input(
-            info, forward, label, pick_ori, noise_cov=noise_csd, rank=rank,
-            pca=False, **depth)
+    (
+        is_free_ori,
+        info,
+        proj,
+        vertices,
+        G,
+        whitener,
+        nn,
+        orient_std,
+    ) = _prepare_beamformer_input(
+        info,
+        forward,
+        label,
+        pick_ori,
+        noise_cov=noise_csd,
+        rank=rank,
+        pca=False,
+        **depth,
+    )
 
     # Compute ranks
     csd_int_rank = []
     if not allow_mismatch:
         noise_rank = compute_rank(noise_csd, info=info, rank=rank)
     for i in range(len(frequencies)):
-        csd_rank = compute_rank(csd.get_data(index=i, as_cov=True),
-                                info=info, rank=rank)
+        csd_rank = compute_rank(
+            csd.get_data(index=i, as_cov=True), info=info, rank=rank
+        )
         if not allow_mismatch:
             for key in csd_rank:
                 if key not in noise_rank or csd_rank[key] != noise_rank[key]:
-                    raise ValueError('%s data rank (%s) did not match the '
-                                     'noise rank (%s)'
-                                     % (key, csd_rank[key],
-                                        noise_rank.get(key, None)))
+                    raise ValueError(
+                        "%s data rank (%s) did not match the "
+                        "noise rank (%s)"
+                        % (key, csd_rank[key], noise_rank.get(key, None))
+                    )
         csd_int_rank.append(sum(csd_rank.values()))
 
     del noise_csd
-    ch_names = list(info['ch_names'])
+    ch_names = list(info["ch_names"])
 
-    logger.info('Computing DICS spatial filters...')
+    logger.info("Computing DICS spatial filters...")
     Ws = []
     max_oris = []
     for i, freq in enumerate(frequencies):
         if n_freqs > 1:
-            logger.info('    computing DICS spatial filter at '
-                        f'{round(freq, 2)} Hz ({i + 1}/{n_freqs})')
+            logger.info(
+                "    computing DICS spatial filter at "
+                f"{round(freq, 2)} Hz ({i + 1}/{n_freqs})"
+            )
 
         Cm = csd.get_data(index=i)
 
@@ -228,29 +269,51 @@ def make_dics(info, forward, csd, reg=0.05, noise_csd=None, label=None,
         # compute spatial filter
         n_orient = 3 if is_free_ori else 1
         W, max_power_ori = _compute_beamformer(
-            G, Cm, reg, n_orient, weight_norm, pick_ori, reduce_rank,
-            rank=csd_int_rank[i], inversion=inversion, nn=nn,
-            orient_std=orient_std, whitener=whitener)
+            G,
+            Cm,
+            reg,
+            n_orient,
+            weight_norm,
+            pick_ori,
+            reduce_rank,
+            rank=csd_int_rank[i],
+            inversion=inversion,
+            nn=nn,
+            orient_std=orient_std,
+            whitener=whitener,
+        )
         Ws.append(W)
         max_oris.append(max_power_ori)
 
     Ws = np.array(Ws)
-    if pick_ori == 'max-power':
+    if pick_ori == "max-power":
         max_oris = np.array(max_oris)
     else:
         max_oris = None
 
-    src_type = _get_src_type(forward['src'], vertices)
+    src_type = _get_src_type(forward["src"], vertices)
     subject = _subject_from_forward(forward)
-    is_free_ori = is_free_ori if pick_ori in [None, 'vector'] else False
+    is_free_ori = is_free_ori if pick_ori in [None, "vector"] else False
     n_sources = np.sum([len(v) for v in vertices])
 
     filters = Beamformer(
-        kind='DICS', weights=Ws, csd=csd, ch_names=ch_names, proj=proj,
-        vertices=vertices, n_sources=n_sources, subject=subject,
-        pick_ori=pick_ori, inversion=inversion, weight_norm=weight_norm,
-        src_type=src_type, source_nn=forward['source_nn'].copy(),
-        is_free_ori=is_free_ori, whitener=whitener, max_power_ori=max_oris)
+        kind="DICS",
+        weights=Ws,
+        csd=csd,
+        ch_names=ch_names,
+        proj=proj,
+        vertices=vertices,
+        n_sources=n_sources,
+        subject=subject,
+        pick_ori=pick_ori,
+        inversion=inversion,
+        weight_norm=weight_norm,
+        src_type=src_type,
+        source_nn=forward["source_nn"].copy(),
+        is_free_ori=is_free_ori,
+        whitener=whitener,
+        max_power_ori=max_oris,
+    )
 
     return filters
 
@@ -263,7 +326,7 @@ def _prepare_noise_csd(csd, noise_csd, real_filter):
             noise_csd = noise_csd.mean()
         noise_csd = noise_csd.get_data(as_cov=True)
         if real_filter:
-            noise_csd['data'] = noise_csd['data'].real
+            noise_csd["data"] = noise_csd["data"].real
     return csd, noise_csd
 
 
@@ -275,10 +338,10 @@ def _apply_dics(data, filters, info, tmin, tfr=False):
     else:
         one_epoch = False
 
-    Ws = filters['weights']
+    Ws = filters["weights"]
     one_freq = len(Ws) == 1
 
-    subject = filters['subject']
+    subject = filters["subject"]
     # compatibility with 0.16, add src_type as None if not present:
     filters, warn_text = _check_src_type(filters)
 
@@ -288,35 +351,41 @@ def _apply_dics(data, filters, info, tmin, tfr=False):
 
         # Apply SSPs
         if not tfr:  # save computation, only compute once
-            M_w = _proj_whiten_data(M, info['projs'], filters)
+            M_w = _proj_whiten_data(M, info["projs"], filters)
 
         stcs = []
         for j, W in enumerate(Ws):
-
             if tfr:  # must compute for each frequency
-                M_w = _proj_whiten_data(M[:, j], info['projs'], filters)
+                M_w = _proj_whiten_data(M[:, j], info["projs"], filters)
 
             # project to source space using beamformer weights
             sol = np.dot(W, M_w)
 
-            if filters['is_free_ori'] and filters['pick_ori'] != 'vector':
-                logger.info('combining the current components...')
+            if filters["is_free_ori"] and filters["pick_ori"] != "vector":
+                logger.info("combining the current components...")
                 sol = combine_xyz(sol)
 
-            tstep = 1.0 / info['sfreq']
+            tstep = 1.0 / info["sfreq"]
 
-            stcs.append(_make_stc(sol, vertices=filters['vertices'],
-                                  src_type=filters['src_type'], tmin=tmin,
-                                  tstep=tstep, subject=subject,
-                                  vector=(filters['pick_ori'] == 'vector'),
-                                  source_nn=filters['source_nn'],
-                                  warn_text=warn_text))
+            stcs.append(
+                _make_stc(
+                    sol,
+                    vertices=filters["vertices"],
+                    src_type=filters["src_type"],
+                    tmin=tmin,
+                    tstep=tstep,
+                    subject=subject,
+                    vector=(filters["pick_ori"] == "vector"),
+                    source_nn=filters["source_nn"],
+                    warn_text=warn_text,
+                )
+            )
         if one_freq:
             yield stcs[0]
         else:
             yield stcs
 
-    logger.info('[done]')
+    logger.info("[done]")
 
 
 @verbose
@@ -413,19 +482,19 @@ def apply_dics_epochs(epochs, filters, return_generator=False, verbose=None):
     """
     _check_reference(epochs)
 
-    if len(filters['weights']) > 1:
+    if len(filters["weights"]) > 1:
         raise ValueError(
-            'This function only works on DICS beamformer weights that have '
-            'been computed for a single frequency. When calling make_dics(), '
-            'make sure to use a CSD object with only a single frequency (or '
-            'frequency-bin) defined.'
+            "This function only works on DICS beamformer weights that have "
+            "been computed for a single frequency. When calling make_dics(), "
+            "make sure to use a CSD object with only a single frequency (or "
+            "frequency-bin) defined."
         )
 
     info = epochs.info
     tmin = epochs.times[0]
 
     sel = _check_channels_spatial_filter(epochs.ch_names, filters)
-    data = epochs.get_data()[:, sel, :]
+    data = epochs.get_data(sel)
 
     stcs = _apply_dics(data=data, filters=filters, info=info, tmin=tmin)
 
@@ -436,8 +505,7 @@ def apply_dics_epochs(epochs, filters, return_generator=False, verbose=None):
 
 
 @verbose
-def apply_dics_tfr_epochs(epochs_tfr, filters, return_generator=False,
-                          verbose=None):
+def apply_dics_tfr_epochs(epochs_tfr, filters, return_generator=False, verbose=None):
     """Apply Dynamic Imaging of Coherent Sources (DICS) beamformer weights.
 
     Apply Dynamic Imaging of Coherent Sources (DICS) beamformer weights
@@ -466,22 +534,23 @@ def apply_dics_tfr_epochs(epochs_tfr, filters, return_generator=False,
     apply_dics
     apply_dics_epochs
     apply_dics_csd
-    """ # noqa E501
+    """  # noqa E501
     _validate_type(epochs_tfr, EpochsTFR)
     _check_tfr_complex(epochs_tfr)
 
-    if filters['pick_ori'] == 'vector':
-        warn('Using a vector solution to compute power will lead to '
-             'inaccurate directions (only in the first quadrent) '
-             'because power is a strictly positive (squared) metric. '
-             'Using singular value decomposition (SVD) to determine '
-             'the direction is not yet supported in MNE.')
+    if filters["pick_ori"] == "vector":
+        warn(
+            "Using a vector solution to compute power will lead to "
+            "inaccurate directions (only in the first quadrent) "
+            "because power is a strictly positive (squared) metric. "
+            "Using singular value decomposition (SVD) to determine "
+            "the direction is not yet supported in MNE."
+        )
 
     sel = _check_channels_spatial_filter(epochs_tfr.ch_names, filters)
     data = epochs_tfr.data[:, sel, :, :]
 
-    stcs = _apply_dics(data, filters, epochs_tfr.info,
-                       epochs_tfr.tmin, tfr=True)
+    stcs = _apply_dics(data, filters, epochs_tfr.info, epochs_tfr.tmin, tfr=True)
     if not return_generator:
         stcs = [[stc for stc in tfr_stcs] for tfr_stcs in stcs]
     return stcs
@@ -531,12 +600,12 @@ def apply_dics_csd(csd, filters, verbose=None):
     ----------
     .. footbibliography::
     """  # noqa: E501
-    ch_names = filters['ch_names']
-    vertices = filters['vertices']
-    n_orient = 3 if filters['is_free_ori'] else 1
-    subject = filters['subject']
-    whitener = filters['whitener']
-    n_sources = filters['n_sources']
+    ch_names = filters["ch_names"]
+    vertices = filters["vertices"]
+    n_orient = 3 if filters["is_free_ori"] else 1
+    subject = filters["subject"]
+    whitener = filters["whitener"]
+    n_sources = filters["n_sources"]
 
     # If CSD is summed over multiple frequencies, take the average frequency
     frequencies = [np.mean(dfreq) for dfreq in csd.frequencies]
@@ -547,27 +616,37 @@ def apply_dics_csd(csd, filters, verbose=None):
     # Ensure the CSD is in the same order as the weights
     csd_picks = [csd.ch_names.index(ch) for ch in ch_names]
 
-    logger.info('Computing DICS source power...')
+    logger.info("Computing DICS source power...")
     for i, freq in enumerate(frequencies):
         if n_freqs > 1:
-            logger.info('    applying DICS spatial filter at '
-                        f'{round(freq, 2)} Hz ({i + 1}/{n_freqs})')
+            logger.info(
+                "    applying DICS spatial filter at "
+                f"{round(freq, 2)} Hz ({i + 1}/{n_freqs})"
+            )
 
         Cm = csd.get_data(index=i)
         Cm = Cm[csd_picks, :][:, csd_picks]
-        W = filters['weights'][i]
+        W = filters["weights"][i]
 
         # Whiten the CSD
         Cm = np.dot(whitener, np.dot(Cm, whitener.conj().T))
 
         source_power[:, i] = _compute_power(Cm, W, n_orient)
 
-    logger.info('[done]')
+    logger.info("[done]")
 
     # compatibility with 0.16, add src_type as None if not present:
     filters, warn_text = _check_src_type(filters)
 
-    return (_make_stc(source_power, vertices=vertices,
-                      src_type=filters['src_type'], tmin=0., tstep=1.,
-                      subject=subject, warn_text=warn_text),
-            frequencies)
+    return (
+        _make_stc(
+            source_power,
+            vertices=vertices,
+            src_type=filters["src_type"],
+            tmin=0.0,
+            tstep=1.0,
+            subject=subject,
+            warn_text=warn_text,
+        ),
+        frequencies,
+    )
